# Copyright (C) 2012 Johan Berggren.
# This file is part of foorep
# See the file 'LICENSE.txt' for copying permission.

from datetime import datetime

from foorep import Plugin

class Pe(Plugin):
    def _overview(self, pe, pefile):
        return {
                "image_base": hex(pe.OPTIONAL_HEADER.ImageBase),
                "entry_point": hex(pe.OPTIONAL_HEADER.AddressOfEntryPoint),
                "machine_type": pefile.MACHINE_TYPE[pe.FILE_HEADER.Machine],
                "dll": pe.FILE_HEADER.IMAGE_FILE_DLL,
                "subsystem": pefile.SUBSYSTEM_TYPE[pe.OPTIONAL_HEADER.Subsystem],
                "timestamp": datetime.fromtimestamp(pe.FILE_HEADER.TimeDateStamp),
                "number_of_rva_and_sizes": pe.OPTIONAL_HEADER.NumberOfRvaAndSizes
            }

    def _imports(self, pe, pefile):
        ret = {}
        if not hasattr(pe, 'DIRECTORY_ENTRY_IMPORT'):
            return None
        for lib in pe.DIRECTORY_ENTRY_IMPORT:
            l = []
            for imp in lib.imports:
                if (imp.name != None) and (imp.name != ""):
                    l.append(imp.name)
            ret[lib.dll.replace(".", "_")] = l
        return ret
        
    def analyze(self, path):
        try:
            import pefile
        except ImportError:
            return None
        try:
            pe = pefile.PE(path)
        except pefile.PEFormatError:
            return None
        result = {
            "type": "pe",
            "name": "PEfile",
            "annotation": {
                    "overview": self._overview(pe, pefile),
                    "imports": self._imports(pe, pefile)
                }
        }
        return result




# Copyright (C) 2012 Johan Berggren.
# This file is part of foorep
# See the file 'LICENSE.txt' for copying permission.

import cherrypy
from bson import json_util
import foorep

class FileResource:
    def __init__(self, repo):
        self.repo = repo
    exposed = True

    def GET(self, uuid):
        cherrypy.response.headers['Content-Type'] = 'application/json'
        doc = self.repo.get(uuid)
        return json_util.dumps(doc)

class ApiRoot:
    def __init__(self):
        self.repo = foorep.Repository()

def main():
    engine = cherrypy.engine
    root = Root()
    root.file = FileResource(root.repo)
    conf = {
        'global': {
            'server.socket_host': "127.0.0.1",
            'server.socket_port': 4781,
            'server.thread_pool': 200,
            'server.socket_queue_size': 60,
            },
        }
    app_conf = {'/': {
            'request.dispatch': cherrypy.dispatch.MethodDispatcher(),
        }
    }
    cherrypy.config.update(conf)
    cherrypy.tree.mount(root, '/api/v1', config=app_conf)
    engine.start()

if __name__ == "__main__":
    main()



# Copyright (C) 2012 Johan Berggren.
# This file is part of foorep
# See the file 'LICENSE.txt' for copying permission.

import os
import sys
import imp
import glob
import uuid
import hashlib
import datetime

import pymongo
import gridfs

try:
    import magic
except ImportError:
    magic = None

class Repository:
    """Repository for forensic artifacts"""
    
    def __init__(self, host='127.0.0.1', port=27017, 
                 database='foorep', collection='repository'):
        """Creates a new or use an existing mongodb database and collection. 
        Sets up an instance of GridFS for storing files.

        :Parameters:
          - `host` MongoDB host to connect to
          - `port` MongoDB port
          - `database` Name of the MongoDB database
          - `collection` Name of the collection
        """
        try:
            self.db = pymongo.MongoClient(host=host, port=port)[database]
            self.collection = self.db[collection]
            self.fs = gridfs.GridFS(self.db)
        except pymongo.errors.ConnectionFailure:
            print("ERROR: Could not connect to collection %s on database %s" % (
                    database, 
                    collection))
            sys.exit()
        # find plugins and import them
        plugin_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)),"plugins")
        for path in glob.glob(os.path.join(plugin_dir,'[!_]*.py')): 
            name, ext = os.path.splitext(os.path.basename(path))
            imp.load_source(name, path)
        self.plugins = Plugin.get_plugins()
    
    def _create_document(self, file=None, filepath=None, filename=None, gridfile=None):
        """Creates a dictionary based on the info of a file.

        :Parameters:
          - `fh` File-like object with read() function
        """
        stat = os.stat(filepath)
        if magic:
            try:
                filetype = magic.from_file(filepath)
            except AttributeError:
                filetype = 'N/A'
        else:
            filetype = 'N/A'

        document = {
            "uuid": uuid.uuid4().hex,
            "file": gridfile,
            "user": None,
            "meta": {
                "filename": filename,
                "filetype": filetype,
                "filesize": stat.st_size,
                "hash": {
                    "md5": hashlib.md5(file).hexdigest(),
                    "sha1": hashlib.sha1(file).hexdigest(),
                    "sha256": hashlib.sha256(file).hexdigest(),
                    "sha512": hashlib.sha512(file).hexdigest(),
                    "ssdeep": None
                    },
                },
            "annotations": {},
            "created": datetime.datetime.now(),
            "updated": datetime.datetime.now()
            }
        return document

    def insert(self, fh, filename=None):
        """Insert information about file and the file itself to the repository.

        :Parameters:
          - `uri` Either a path to a local file or a URL to download
        """
        try:
            filebuffer = fh.read()
        except AttributeError:
            fh = open(fh, "rb")
            filebuffer = fh.read()
        filepath = fh.name
        if not filename:
            filename = os.path.basename(filepath)

        gridfile = self.fs.put(filebuffer,
                filename=filename)
        document = self._create_document(file=filebuffer, 
                filepath=filepath,
                filename=filename,
                gridfile=gridfile)
        self.collection.insert(document)
        for plugin in self.plugins:
            res = plugin.analyze(filepath)
            if res:
                self.annotate(document['uuid'], res)
        return document

    def remove(self, id):
        """Removes a document+file from the repository.

        :Parameters:
          - `id` MongoDB ObjectID or uuid
        """
        try:
            doc = self.get(id)
            file = doc.get('file')
        except:
            return None
        if file:
            if self.fs.exists(file):
                self.fs.delete(file)
        self.collection.remove(doc['_id'])
        return True 

    def annotate(self, id, annotation):
        """Add annotation to document.

        :Parameters:
          - `id` MongoDB ObjectID or uuid
          - `annotation` Dictionary with annotation
        """
        document = self.get(id)
        annotations = document.get('annotations')
        new_annotation = annotations.setdefault(annotation.get('type'), [])
        new_annotation.append({"created": datetime.datetime.now(), "annotation": annotation.get('annotation')})
        self.collection.save(document)
        return True

    def get(self, id):
        """Get document.

        :Parameters:
          - `id` MongoDB ObjectID or uuid
        """
        document = self.collection.find_one({'$or': [
            {"_id": id },
            {"uuid": id }
        ]})
        return document

    def get_file(self, id):
        """Get filehandler

        :Parameters:
          - `id` MongoDB (gridFS) ObjectID
        """
        return self.fs.get(id)

    def search(self, q):
        """Search for document.

        :Parameters:
          - `hash` md5, sha1, sha256 or sha512 to search for
        """
        result = []
        query = self.collection.find({'$or': [
            {"meta.hash.md5":       q},
            {"meta.hash.sha1":      q},
            {"meta.hash.sha256":    q},
            {"meta.hash.sha512":    q},
            {"uuid":                q},
            {"meta.filename":       {'$regex': q}}
        ]})
        for sample in query:
            result.append(sample)
        return result

    def list(self, limit=0):
        return self.collection.find().limit(limit)

class PluginMount(type):
    def __init__(self, name, bases, attrs):
        if not hasattr(self, 'plugins'):
            self.plugins = []
        else:
            self.plugins.append(self)
    def get_plugins(self, *args, **kwargs):
        return [p(*args, **kwargs) for p in self.plugins]

class Plugin:
    __metaclass__ = PluginMount

if __name__ == '__main__':
    pass


# Copyright (C) 2012 Johan Berggren.
# This file is part of foorep
# See the file 'LICENSE.txt' for copying permission.

import cherrypy
import jinja2
import foorep
import tempfile
import os
import base64
import re

from foorep.restapi import ApiRoot, FileResource

site_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)),"site")

def guess_autoescape(template_name):
    if template_name is None or '.' not in template_name:
        return False
    ext = template_name.rsplit('.', 1)[1]
    return ext in ('html', 'htm', 'xml')

env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(os.path.join(site_dir, 'templates')),
        autoescape=guess_autoescape,
        extensions=['jinja2.ext.autoescape'])

def datetimeformat(value, format='%Y-%m-%d %H:%M:%S'):
    return value.strftime(format)

env.filters['datetimeformat'] = datetimeformat

def is_picture(file):
    if re.match('JPEG|PNG', file['meta']['filetype']):
        return True
    return False

class App:
    @cherrypy.expose
    def index(self):
        tmpl = env.get_template('index.html')
        return tmpl.render()

class Search:
    def __init__(self, repo):
        self.repo = repo
    @cherrypy.expose
    def index(self, q=None):
        tmpl = env.get_template('result.html')
        if not q or q == '.':
            raise cherrypy.HTTPRedirect("/")
        result = self.repo.search(q)
        return tmpl.render(result=result)

class Annotate:
    def __init__(self, repo):
        self.repo = repo
    @cherrypy.expose
    def default(self, uuid, type, value):
        annotation = {"type": type, "annotation": value}
        self.repo.annotate(uuid, annotation)
        raise cherrypy.HTTPRedirect("/file/%s" % uuid)

class File:
    def __init__(self, repo):
        self.repo = repo
    @cherrypy.expose
    def default(self, uuid):
        tmpl = env.get_template('file.html')
        file = self.repo.get(uuid)
        if is_picture(file):
            img = self.repo.get_file(file['file'])
            b64img = base64.b64encode(img.read())
        else:
            b64img = None
        return tmpl.render(file=file, b64img=b64img)

class Upload:
    def __init__(self, repo):
        self.repo = repo
    @cherrypy.expose
    def default(self, fh):
        _tmpf = tempfile.mkstemp()
        tmpfn = _tmpf[1]
        tmpfh = os.fdopen(_tmpf[0], "wb")
        tmpfh.write(fh.file.read())
        tmpfh.close()
        doc = self.repo.insert(tmpfn, filename=fh.filename)
        os.remove(tmpfn)
        raise cherrypy.HTTPRedirect("/file/%s" % doc['uuid'])

class Download:
    def __init__(self, repo):
        self.repo = repo
    @cherrypy.expose
    def default(self, uuid):
        doc = self.repo.get(uuid)
        fh = self.repo.get_file(doc['file'])
        return cherrypy.lib.static.serve_fileobj(fh, content_type="application/x-download",
                disposition="attachment", name=doc['meta']['hash']['sha1'])

def main():
    repo = foorep.Repository()
    engine = cherrypy.engine
    global_conf = {
        'global': {
            'server.socket_host': '127.0.0.1',
            'server.socket_port': 4780,
            'server.thread_pool': 200,
            'server.socket_queue_size': 60,
            },
    }
    cherrypy.config.update(global_conf)

    # Start WebUI
    webui = App()
    webui.search = Search(repo)
    webui.file = File(repo)
    webui.download = Download(repo)
    webui.upload = Upload(repo)
    webui.annotate = Annotate(repo)
    webui_conf = {
        '/': {
        },
        '/static': {
            'tools.staticdir.on': True,
            'tools.staticdir.root': site_dir,
            'tools.staticdir.dir': 'static',
        }
    }
    # Start RESTFul API
    restapi = ApiRoot()
    restapi.file = FileResource(repo)  
    restapi_conf = {
            '/': {
                'request.dispatch': cherrypy.dispatch.MethodDispatcher(),
            }
    }

    cherrypy.tree.mount(webui, '/', config=webui_conf)
    cherrypy.tree.mount(restapi, '/api/v1', config=restapi_conf)
    engine.start()
    engine.block()
    
if __name__ == "__main__":
    main()



import unittest
import pymongo
from foorep import Repository
import uuid

class FoorepTest(unittest.TestCase):
    def setUp(self):
        """ Setup some initial configuration, and initiate a foorep repo object """
        self.dbname = uuid.uuid4().hex
        self.repo = Repository(database=self.dbname)

    def test_insert(self):
        """Add file to repository, check if return a dict """
        self.assertIsInstance(self.repo.insert(__file__), dict)

    def test_list(self):
        """List samples in repository"""
        self.assertIsInstance(self.repo.list(), pymongo.cursor.Cursor)
    
    def tearDown(self):
        c = pymongo.Connection()
        c.drop_database(self.dbname)

if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2012 Johan Berggren.
# This file is part of foorep
# See the file 'LICENSE.txt' for copying permission.

from foorep import Plugin

class Exif(Plugin):
    def analyze(self, path):
        try:
            import pyexiv2
        except ImportError:
            return None
        try:
            exif = pyexiv2.ImageMetadata(path)
            exif.read()
        except IOError:
            return None
        except AttributeError:
            return None
        self.tags = exif.exif_keys
        result = {
            "type": "exif",
            "name": "EXIF",
            "annotation": {},
        }
        if self.tags:
            for tag in exif.exif_keys:
                result['annotation'][tag.replace(".", "_")] = exif[tag].human_value
        return result 



from distutils.core import setup

setup(
    name='foorep',
    version='0.1.3-2',
    author='Johan Berggren',
    author_email='jbn@klutt.se',
    packages=['foorep', 'foorep.test'],
    url='http://foorensics.blogspot.com',
    license='LICENSE.txt',
    description='Malware Repository for humans',
    long_description=open('README.md').read(),
    install_requires=[
        "pymongo==2.4",
        "CherryPy==3.2.2",
        "Jinja2==2.6",
        "pefile==1.2.10-123",
        "python-magic==0.4.3",
    ],
    include_package_data=True,
    package_data = {
        'foorep': [
            'plugins/*',
            'site/static/js/*',
            'site/static/css/*',
            'site/static/img/*',
            'site/static/bootstrap/js/*',
            'site/templates/plugins/*',
            'site/templates/*.html',
            'site/static/bootstrap/css/*',
            'site/static/bootstrap/img/*']
        },
        entry_points={
            'console_scripts':
                ['foorep=foorep.cli:main', 'foorepd=foorep.web:main']
        },
)



# Copyright (C) 2012 Johan Berggren.
# This file is part of foorep
# See the file 'LICENSE.txt' for copying permission.

"""
foorep is a malware repository
"""


import foorep
from bson import json_util
import argparse
import json

def main():
    """
    The main entry point for the foorep cli tool
    """
    def add(args, repo):
        try:
            doc = repo.insert(args.path)
        except IOError:
            print('File not found')
            return
        if args.verbose:
            print('%s added to repository with id %s' %
                    (doc['meta']['filename'], doc['uuid']))
        return

    def remove(args, repo):
        result = repo.remove(args.uuid)
        if not result:
            print('No such file in the repository')
        return

    def search(args, repo):
        for malware in repo.search(args.hash):
            print malware['created'].strftime('%Y-%m-%d %H:%M:%S'), malware['uuid'], malware['meta']['filename']
        return

    def dump(args, repo):
        doc = repo.get(args.uuid)
        if not doc:
            print('No such file in the repository')
        else:
            print(json.dumps(doc, indent=1, default=json_util.default))
        return

    def annotate(args, repo):
        annotation = {"type": args.type, "value":args.message}
        repo.annotate(args.uuid, annotation)
        return

    def list(args, repo):
        if args.limit:
            limit = int(args.limit)
        else:
            limit = 10
        for malware in repo.list(limit=limit):
            print malware['created'].strftime('%Y-%m-%d %H:%M:%S'), malware['uuid'], malware['meta']['filename'] 

    repo = foorep.Repository()
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--verbose', action='store_true', help='verbose')
    subparsers = parser.add_subparsers(dest='command', title='Commands',
            description='These are the commands that I know of..')
    parser_add = subparsers.add_parser('add', help='Add malware to repository')
    parser_add.add_argument('path', help='Path to file')
    parser_add.set_defaults(func=add)
    parser_remove = subparsers.add_parser('remove', help='Remove malware from repository')
    parser_remove.add_argument('uuid', help='File to remove from the repository')
    parser_remove.set_defaults(func=remove)
    parser_search = subparsers.add_parser('search', help='Search for malware in repository')
    parser_search.add_argument('hash', help='Hash to search for')
    parser_search.set_defaults(func=search)
    parser_dump = subparsers.add_parser('dump', help='Dump raw JSON document for malware')
    parser_dump.add_argument('uuid', help='File to dump')
    parser_dump.set_defaults(func=dump)
    parser_annotate = subparsers.add_parser('annotate', help='Add annotation to malware')    
    parser_annotate.add_argument('uuid', help='File to annotate')
    parser_annotate.add_argument('-t', '--type', metavar='TYPE', help='Type of annotation')
    parser_annotate.add_argument('-m', '--message', metavar='VALUE', help='The content of the annotation')
    parser_annotate.set_defaults(func=annotate)
    parser_list = subparsers.add_parser('list', help='List malware in repository')
    parser_list.add_argument('-l', '--limit', help='Limit amount of records returned, default is 10')
    parser_list.set_defaults(func=list)
    args = parser.parse_args()
    args.func(args, repo)

if __name__ == '__main__':
    main()
